﻿###############################################################################
#                   Deleting all security groups for Office 365               #
#                                  V3.9.8.9                                   #
#                       (C)2006-2022 KoXo Developpement                       #
###############################################################################
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Tools.ps1")
.("%LAUNCHING_PATH%\AdminScripts\KoXo-MSol.ps1")
###############################################################################
#                                    Main                                     #
###############################################################################
$Copyright = "(C)2006-2022 KoXo Developpement"
$Version = "V3.9.8.9"
$Title = "Deleting all security groups for Office 365 $($Version)", "$($Copyright)"
# Office 365 params
$Office365LoginName = "%OFFICE365_ADMIN%"
$Office365LoginPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
# Proxy parameters
$ProxyEnabled = $#BOOL{%PROXY_ENABLED%}
$ProxyUser = "%PROXY_USER%"     
$ProxyPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
$ProxyUrl = "%PROXY_ADDRESS%"
$ProxyPort = %PROXY_PORT%
# Log filename
$LogFile = $MyInvocation.MyCommand.Definition 
$LogFile = $LogFile -replace(".ps1",".log")
#
$global:total_errors_count = 0
#
banner $Title
check_version
if (import_modules MSOnline) {
    set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
    if ((connect_msol $Office365LoginName $Office365LoginPassword) -eq $true) {
        $Groups = Get-MsolGroup -GroupType "Security" -ErrorAction SilentlyContinue
        if ($Groups -ne $null){  
            write-host "Security groups are : " -foregroundcolor "white"
            ForEach ($Group in $Groups) {
                Write-host -nonewline """$($Group.DisplayName)"" "
            }
            write-host        
            write-host -nonewline "Delete all these groups ? Enter YES to continue : " -foregroundcolor "magenta"
            $ret = read-host
            $ret = $ret.ToLower()
            if ($ret -eq "yes") {  
                ForEach ($Group in $Groups) {
                    delete_security_group $Group.ObjectId $Group.DisplayName $Group.Description
                }
            }     
        }
        else {
            write-host -nonewline "There are no security groups ! Press any key to continue : " -foregroundcolor "magenta"
            read-host
        }   
    }
    else {
        $global:total_errors_count++
        $S = "Unable to connect to MSOnline !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
    }
}
else {
    $global:total_errors_count++
    $S = "ExchangeOnlineManagement Powershell module cannot be loaded !"
    Write-Host $S -Foregroundcolor "Red"
    Log_Message "ERROR" $S
}
log_total_errors_count