﻿###############################################################################
#           Create/Modify Dynamic Distribution Group for Office 365           #
#                             For Secondary Group                             #
#                                  V3.9.8.9                                   #
#                       (C)2006-2022 KoXo Developpement                       #
###############################################################################
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Tools.ps1")
.("%LAUNCHING_PATH%\AdminScripts\KoXo-ExchangeOnline.ps1")
###############################################################################
#                                    Main                                     #
###############################################################################
$Copyright = "(C)2006-2022 KoXo Developpement"
$Version = "V3.9.8.9"
$Title = "Create/Modify dynamic distribution group for Office 365 $($Version)", "$($Copyright)"
# Office 365 params
$Office365LoginName = "%OFFICE365_ADMIN%"
$Office365LoginPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
# Proxy parameters
$ProxyEnabled = $#BOOL{%PROXY_ENABLED%}
$ProxyUser = "%PROXY_USER%"     
$ProxyPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
$ProxyUrl = "%PROXY_ADDRESS%"
$ProxyPort = %PROXY_PORT%
# Only users from domain can write messages
$RequireSenderAuthenticationEnabled = $#BOOL{%OFFICE365_SENDER_AUTHENTICATION%}
# Log filename
$LogFile = $MyInvocation.MyCommand.Definition 
$LogFile = $LogFile -replace(".ps1",".log")
#
$global:total_errors_count = 0
#
banner $Title
check_version
if (import_modules ExchangeOnlineManagement) {
    set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
    if ((connect_exo $Office365LoginName $Office365LoginPassword) -eq $true) {
        if ($RequireSenderAuthenticationEnabled) {
            write-host "Only domain's users can write messages" -foregroundcolor "magenta"
        }
        else {
            write-host "Everyone can write messages" -foregroundcolor "magenta"
        }
        ### BEGIN_REPETITION ###
        $global:errors_count = 0
        # KoXo Administrator's variables
        $DynamicDistributionGroupName = "%SECONDARY_GROUP%"
        $DynamicDistributionGroupDisplayName = "%SECONDARY_GROUP_DESC%"
        $DynamicDistributionGroupAlias = "#LOWER{#REMOVE_SPACE{%SECONDARY_GROUP%}}"
        $DynamicDistributionGroupEmail = "%SECONDARY_GROUP_EMAIL%"
        $DynamicDistributionGroupCustomAttribute = "#REMOVE_SPACE{%SECONDARY_GROUP%}"
        log_message "OFFICE365" "$($DynamicDistributionGroupDisplayName) ($($DynamicDistributionGroupEmail))"
        $S = "Dynamic distribution group : $($DynamicDistributionGroupAlias)"
        write-host $S
        if (dynamic_group_exists($DynamicDistributionGroupAlias)) {
            modify_dynamic_distribution_group $DynamicDistributionGroupName $DynamicDistributionGroupDisplayName $DynamicDistributionGroupAlias $DynamicDistributionGroupEmail $RequireSenderAuthenticationEnabled $DynamicDistributionGroupCustomAttribute
        }
        else {
            create_dynamic_distribution_group $DynamicDistributionGroupName $DynamicDistributionGroupDisplayName $DynamicDistributionGroupAlias $DynamicDistributionGroupEmail $RequireSenderAuthenticationEnabled $DynamicDistributionGroupCustomAttribute 
        }
        log_errors_count $DynamicDistributionGroupName
        $global:total_errors_count = $global:total_errors_count + $global:errors_count
        ### END_REPETITION ###
		Disconnect-ExchangeOnline -Confirm:$false
    }
    else {
        $global:total_errors_count++
        $S = "Unable to connect to Exchange Online !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
    }
}
else {
    $global:total_errors_count++
    $S = "ExchangeOnlineManagement Powershell module cannot be loaded !"
    Write-Host $S -Foregroundcolor "Red"
    Log_Message "ERROR" $S
}
log_total_errors_count