﻿###############################################################################
#            Install Office 365 modules for Exchange, MSOnline, Teams         #
#                                  V3.9.8.9                                   #
#                       (C)2006-2022 KoXo Developpement                       #
###############################################################################
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Tools.ps1")
###############################################################################
function display_system_informations {
    # Windows Management Framework Version
    $WMFVersion = get_wmf_version
    $WindowsOSName = get_os_name
    $WindowsOSVersion = get_os_version
    Write-Host "Operating system : " -NoNewline -ForegroundColor White
    Write-Host "$($WindowsOSName) V$($WindowsOSVersion)" -ForegroundColor Yellow
    $S = "Operating system : $($WindowsOSName) V$($WindowsOSVersion)"
    Log_Message "INFO" $S
    Write-Host "Windows Management Framework : " -NoNewline -ForegroundColor White
    Write-Host "V$($WMFVersion)" -ForegroundColor Yellow
    Write-Host -NoNewline -ForegroundColor White
    $S = "Windows Management Framework : V$($WMFVersion)"
    Log_Message "INFO" $S
}
###############################################################################
# https://docs.microsoft.com/fr-fr/azure/active-directory/hybrid/reference-connect-tls-enforcement
function enable_tls12 {
    If (-Not (Test-Path "HKLM:\SOFTWARE\WOW6432Node\Microsoft\.NETFramework\v4.0.30319")) {
        New-Item "HKLM:\SOFTWARE\WOW6432Node\Microsoft\.NETFramework\v4.0.30319" -Force | Out-Null
    }
    New-ItemProperty -Path "HKLM:\SOFTWARE\WOW6432Node\Microsoft\.NETFramework\v4.0.30319" -Name "SystemDefaultTlsVersions" -Value "1" -PropertyType "DWord" -Force | Out-Null
    New-ItemProperty -Path "HKLM:\SOFTWARE\WOW6432Node\Microsoft\.NETFramework\v4.0.30319" -Name "SchUseStrongCrypto" -Value "1" -PropertyType "DWord" -Force | Out-Null
    If (-Not (Test-Path "HKLM:\SOFTWARE\Microsoft\.NETFramework\v4.0.30319")) {
        New-Item "HKLM:\SOFTWARE\Microsoft\.NETFramework\v4.0.30319" -Force | Out-Null
    }
    New-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\.NETFramework\v4.0.30319" -Name "SystemDefaultTlsVersions" -Value "1" -PropertyType "DWord" -Force | Out-Null
    New-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\.NETFramework\v4.0.30319" -Name "SchUseStrongCrypto" -Value "1" -PropertyType "DWord" -Force | Out-Null
    If (-Not (Test-Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server")) {
        New-Item "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server" -Force | Out-Null
    }
    New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server" -Name "Enabled" -Value "1" -PropertyType "DWord" -Force | Out-Null
    New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Server" -Name "DisabledByDefault" -Value "0" -PropertyType "DWord" -Force | Out-Null
    If (-Not (Test-Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client")) {
        New-Item "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client" -Force | Out-Null
    }
    New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client" -Name "Enabled" -Value "1" -PropertyType "DWord" -Force | Out-Null
    New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client" -Name "DisabledByDefault" -Value "0" -PropertyType "DWord" -Force | Out-Null
    Write-Host "TLS 1.2 has been enabled. You must restart the Windows Server for the changes to take affect." -ForegroundColor Cyan
}
###############################################################################
function set_security_protocols {
    $S = "Tls, Tls11, Tls12"
    [Net.ServicePointManager]::SecurityProtocol = $S
    $S = "Security protocol set to : $($S)"
    Write-Host $S -ForegroundColor Cyan
    Log_Message "INFO" $S
}
###############################################################################
function tls12_client_enabled {
    $RegPath = "HKLM:\SYSTEM\CurrentControlSet\Control\SecurityProviders\SCHANNEL\Protocols\TLS 1.2\Client"
    $regValue = Get-ItemProperty -Path $regPath -Name "Enabled" -ErrorAction SilentlyContinue
    if ($RegValue -ne $null) {
        return ($RegValue.Enabled -eq 1)
    }
    else {
        return $false
    }
}
###############################################################################
function update_wmf {
    $WMFVersion = get_wmf_version
    # Not updated Windows 2012 --> V4.0.-1.-1, Actual --> V5.1.22000.282
    $InstallWMF = compare_text_versions $WMFVersion "5.0.0.0"
    If ($InstallWMF -eq -1) {
        enable_tls12 # avoid two restarts
        $OSVersion = get_os_version
        # https://www.microsoft.com/en-us/download/details.aspx?id=54616
        If ($OSVersion -eq "6.2.9200") {
            # Windows 2012 V6.2.9200 -> W2K12-KB3191565-x64.msu
            $Source = "https://go.microsoft.com/fwlink/?linkid=839513"
            $Destination = $env:TEMP + "\W2K12-KB3191565-x64.msu"
        }
        else {
            # Windows 2012R2 V6.3.9600 -> Win8.1AndW2K12R2-KB3191564-x64.msu
            $Source = "https://go.microsoft.com/fwlink/?linkid=839516"
            $Destination = $env:TEMP + "\Win8.1AndW2K12R2-KB3191564-x64.msu"
        }
        Write-Host "Trying to download Windows Management Framework V5.1"
        Write-Host "Url=$($Source)"
        Write-Host "Please, wait ..." -ForegroundColor "Cyan" -NoNewline
        $wc = New-Object System.Net.WebClient
        $wc.DownloadFile($Source, $Destination)
        Write-Host
        $Install = Start-Process $Destination -Wait -PassThru
        if ($Install.ExitCode -eq 0) {
            $S = "Windows Management Framework V5.1 installed successfully"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S   
            Write-Host "You must restart the Windows Server for the changes to take affect." -ForegroundColor Cyan
        }
        else {
            $S = "Installation failed for Windows Management Framework V5.1, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
        } 
        # Delete downloaded file and exit
        $FileExists = Test-Path $Destination
        If ($FileExists) { Get-Item $Destination | Remove-Item } 
        Write-Host "Press any key to exit ..." -ForegroundColor Cyan
        Read-Host
        Exit
    }
}
###############################################################################
function is_nuget_present {
    $Packages = Get-PackageProvider -ListAvailable
    $Nuget = $Packages | Where { $_.Name -eq "Nuget" }
    $Res = ($Nuget -ne $null)
    Return $Res
}
###############################################################################
function install_update_nuget {
    Write-Host "Checking for installed package providers ..." -NoNewline
    $Nuget = is_nuget_present
    Write-Host
    If ($Nuget -eq $false) {
        $error.Clear()
        $PackageProvider = Install-PackageProvider -Name "Nuget" -Force -ErrorAction SilentlyContinue # Oct 2021 : V2.8.5.208
        If ($PackageProvider -ne $null) {
            $S = "Package provider $($PackageProvider.Name) V$($PackageProvider.Version) is now installed"
            Log_Message "INFO" $S
            Write-Host $S -ForegroundColor Green
        }
        else {
            $global:total_errors_count++
            $S = "Cannot install package provider Nuget ! Error=$($error[0])"
            Log_Message "ERROR" $S
            Write-Host $S -ForegroundColor Red
        }    
    }
    else {
        $ActualPackageProvider = Get-PackageProvider -Name "Nuget" -ErrorAction SilentlyContinue
        If ($ActualPackageProvider -ne $null) {
            $S = "Package provider $($ActualPackageProvider.Name) V$($ActualPackageProvider.Version) is already installed"
            Write-Host $S -ForegroundColor Cyan
            # Check available version
            Write-Host "Checking package provider updates ..." -NoNewline
            $error.Clear()
            $LastPackageProvider = Find-PackageProvider -Name "Nuget" -ErrorAction SilentlyContinue    
            Write-Host
            If ($LastPackageProvider -ne $null) {
                $UpdatePackageProvider = compare_text_versions $ActualPackageProvider.Version $LastPackageProvider.Version 
                If ($UpdatePackageProvider -eq -1) {
                    Write-Host "Update Nuget V$($ActualPackageProvider.Version) to V$($LastPackageProvider.Version) ? [Y/N] : " -NoNewline
                    $Response = Read-Host
                    If ($Response.ToLower() -eq "y") {
                        $UpdatedPackageProvider = Install-PackageProvider -Name "Nuget" -MinimumVersion $LastPackageProvider.Version -Force
                        If ($UpdatedPackageProvider -ne $null) {
                            $S = "Package provider Nuget has been updated from V$($ActualPackageProvider.Version) to V$($UpdatedPackageProvider.Version)"
                            Write-Host $S -ForegroundColor Green
                            Log_Message "OK" $S
                        }
                    }
                }
            }
            else {
                $global:total_errors_count++
                $S = "Unable to find Nuget package provider online ! Error=$($error[0])"
                Write-Host $S -ForegroundColor Red
                Log_Message "ERROR" $S
            }
        }
    }
}
###############################################################################
#                                    Main                                     #
###############################################################################
$Copyright = "(C)2006-2022 KoXo Developpement"
$Version = "V3.9.8.9"
$Title = "Install Office 365 modules for Exchange, MSOnline, Teams $($Version)", "$($Copyright)"
# Proxy parameters
$ProxyEnabled = $#BOOL{%PROXY_ENABLED%}
$ProxyUser = "%PROXY_USER%"     
$ProxyPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
$ProxyUrl = "%PROXY_ADDRESS%"
$ProxyPort = %PROXY_PORT%
# Log filename
$LogFile = $MyInvocation.MyCommand.Definition 
$LogFile = $LogFile -replace (".ps1", ".log")
#
$global:total_errors_count = 0
#
banner $Title
display_system_informations
update_wmf
if (-not (tls12_client_enabled)) {
    enable_tls12 
} 
set_security_protocols
set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
install_update_nuget
install_update_module MSOnline
install_update_module ExchangeOnlineManagement
install_update_module MicrosoftTeams
log_total_errors_count