﻿###############################################################################
#                   Create/Modify a team for Microsoft Teams                  #
#                                    V3.9.8.9                                 #
#                       (C)2006-2022 KoXo Developpement                       #
###############################################################################
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Tools.ps1")
.("%LAUNCHING_PATH%\AdminScripts\KoXo-MSol.ps1")
.("%LAUNCHING_PATH%\AdminScripts\KoXo-ExchangeOnline.ps1")
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Teams.ps1")
###############################################################################
#                                    Main                                     #
###############################################################################
$Copyright = "(C)2006-2022 KoXo Developpement"
$Version = "V3.9.8.9"
$Title = "Create/Modify a team for Microsoft Teams $($Version)", "$($Copyright)"
# Office 365 params
$Office365LoginName = "%OFFICE365_ADMIN%"
$Office365LoginPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
# Proxy parameters
$ProxyEnabled = $#BOOL{%PROXY_ENABLED%}
$ProxyUser = "%PROXY_USER%"     
$ProxyPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
$ProxyUrl = "%PROXY_ADDRESS%"
$ProxyPort = %PROXY_PORT%
# Log filename
$LogFile = $MyInvocation.MyCommand.Definition 
$LogFile = $LogFile -replace(".ps1",".log")
#
$global:total_errors_count = 0
$global:total_warnings_count = 0
#

#
banner $Title
check_version
if (import_modules "MicrosoftTeams,MSOnline,ExchangeOnlineManagement") {
    set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
    if ((connect_msol $Office365LoginName $Office365LoginPassword) -eq $true) {
        if ((connect_exo $Office365LoginName $Office365LoginPassword) -eq $true) {
            if ((connect_teams $Office365LoginName $Office365LoginPassword) -eq $true) {
                $MSolDefaultDomain = get_msol_default_domain
                $Office365DNSDomain = "%OFFICE365_DNS_DOMAIN%"
                #
                ### BEGIN_REPETITION ###
                $global:errors_count = 0
                $ResetTeam = (%SECONDARY_GROUP_TEAMS_RESET% -ne 0)                
                $SecondaryGroupDisplayName = "%SECONDARY_GROUP_TEAMS_DISPLAY_NAME%"
                $SecondaryGroup = "%SECONDARYGROUP%"
                $CompanyShortName = "%COMPANY_SHORT_NAME%".Trim()
                if ([String]::IsNullorEmpty($CompanyShortName)) {
                    $MailNickName = -Join ("teams-", $SecondaryGroup.ToLower())
                }
                else {
                    $MailNickName = -Join ("teams-", $CompanyShortName.ToLower(), ".", $SecondaryGroup.ToLower())
                }
                $SecondaryGroupDescription = "%SECONDARY_GROUP_DESC%"
                $Visibility = "%SECONDARY_GROUP_TEAMS_VISIBILITY%"
                $Owners = "%SECONDARY_GROUP_TEAMS_OWNERS%"
                $Members = "%SECONDARY_GROUP_USERS_EMAIL%"
                if (![String]::IsNullorEmpty("%SECONDARY_GROUP_TEAMS_MEMBERS%")) { 
                    $Members = -Join ($Members, ",%SECONDARY_GROUP_TEAMS_MEMBERS%")
                }
                $Template = "%SECONDARY_GROUP_TEAMS_TEMPLATE%"
                $Channels = "%SECONDARY_GROUP_TEAMS_CHANNELS%"	
				if (team_exists($MailNickName)) {
                   $Team = team_modify $SecondaryGroupDisplayName $SecondaryGroupDescription $MailNickName $Visibility $Owners $Office365LoginName $Members $Channels $ResetTeam $MSolDefaultDomain $Office365DNSDomain
				}
				else
				{
                   $Team = team_create $SecondaryGroupDisplayName $SecondaryGroupDescription $MailNickName $Visibility $Owners $Office365LoginName $Template $Channels	$MSolDefaultDomain $Office365DNSDomain
				}
                log_errors_count $SecondaryGroupDisplayName
                $global:total_errors_count = $global:total_errors_count + $global:errors_count
                ### END_REPETITION ###
                #
                Disconnect-MicrosoftTeams -Confirm:$false
                Disconnect-ExchangeOnline -Confirm:$false
            }
            else {
                $global:total_errors_count++
                $S = "Unable to connect to Microsoft Teams !"
                Write-Host $S -Foregroundcolor "Red"
                Log_Message "ERROR" $S
            }
        }
        else {
            $global:total_errors_count++
            $S = "Unable to connect to Exchange Online !"
            Write-Host $S -Foregroundcolor "Red"
            Log_Message "ERROR" $S
        }
    }
    else {
        $global:total_errors_count++
        $S = "Unable to connect to MSOnline !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
    }
}
else {
    $global:total_errors_count++
    $S = "All Powershell modules cannot be loaded !"
    Write-Host $S -Foregroundcolor "Red"
    Log_Message "ERROR" $S
}
log_total_errors_count
log_total_warnings_count