﻿###############################################################################
#                                                                             #
#                 Security group can create security groups                   #
#                     (C)2006-2022 KoXo Developpement                         #
#                                                                             #
###############################################################################
.("%LAUNCHING_PATH%\AdminScripts\KoXo-Tools.ps1")
###############################################################################
function connect_azure_ad {
    Param([string] $MyName, $MyPassword)
    $Cred = get_cred $MyName $MyPassword
    Connect-AzureAD -Credential $Cred -ErrorAction SilentlyContinue
    if ([String]::IsNullorEmpty($error[0])) {
        $S = "Connection was successful to Microsoft AzureAD"
        log_message "OK" $S
        Return $true
    } 
    else {
        $global:total_errors_count++
        $S = "Connection was unsuccessful to Microsoft AzureAD || $($error[0])"
        log_message "ERROR" $S
        # Display error
        Write-host $S -foregroundcolor "red"
        Write-host "Press 'Enter' to continue ..."
        Read-Host   
        return $false
    }   
}
###############################################################################
#                                    Main                                     #
###############################################################################
$Copyright = "(C)2006-2022 KoXo Developpement"
$Version = "V3.9.8.9"
$Title = "Security group can create security groups", $Copyright
# AzureAD
$ModuleName = "AzureADPreview"
$GroupName = "SecurityGroupCreationGroup"                          # <-- Name of security group
$GroupDesc = "Members of this group can create security groups"    # <-- Security group description
# Office 365 params
$Office365LoginName = "%OFFICE365_ADMIN%"
$Office365LoginPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
# Proxy parameters
$ProxyEnabled = $#BOOL{%PROXY_ENABLED%}
$ProxyUser = "%PROXY_USER%"     
$ProxyPassword = [System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
$ProxyUrl = "%PROXY_ADDRESS%"
$ProxyPort = %PROXY_PORT%
# Log filename
$LogFile = $MyInvocation.MyCommand.Definition 
$LogFile = $LogFile -replace (".ps1", ".log")
#
$global:total_errors_count = 0
#
banner $Title
check_version
if (!(Get-Module -Name $ModuleName)) {
    install_update_module $ModuleName
}
if (import_modules $ModuleName) {
    set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
    if ((connect_azure_ad $Office365LoginName $Office365LoginPassword) -eq $true) {
        # Create the security group if necessary
        Write-Host "Security group : " -NoNewline
        Write-Host """$($GroupName)""" -ForegroundColor Yellow -NoNewline
        Write-Host "($($GroupDesc))" -ForegroundColor Yellow
        if ((Get-AzureADGroup -Filter ("DisplayName eq '{0}'" -f $GroupName)) -eq $null) {
            $GrpSecu = New-AzureADGroup -DisplayName $GroupName -Description $GroupDesc -MailEnabled $false -SecurityEnabled $true -MailNickName "NotSet"
            If ($GrpSecu -ne $null) {
                $S = "Security group ""{0}"", id=""{1}"" created !" -f $GrpSecu.DisplayName, $GrpSecu.ObjectId
                Write-Host $S -Foregroundcolor "Green"
                log_message "OK" $S
            }
            else {
                $S = "Cannot create security group ""{0}"" !" -f $GroupName
                Write-Host $S -Foregroundcolor "Red"
                log_message "ERROR" $S
                $global:total_errors_count++
            }
        }
        Else {
            $GrpSecu = Get-AzureADGroup  -Filter ("DisplayName eq '{0}'" -f $GroupName)
            $Members = Get-AzureADGroupMember -All:$true -ObjectId $GrpSecu.ObjectId
            if ($Members -eq $null) {
                Write-Host "Security group " -NoNewline
                Write-Host """$($GrpSecu.DisplayName)"" ($($GrpSecu.ObjectId))" -NoNewline -ForegroundColor Yellow
                Write-Host " exists and is empty."
            }
            else {
                Write-Host "Security group " -NoNewline
                Write-Host """$($GrpSecu.DisplayName)"" ($($GrpSecu.ObjectId))" -NoNewline -ForegroundColor Yellow
                Write-Host " exists and contains : "
                ForEach ($Member in $Members) {
                    Write-Host -nonewline """$($Member.DisplayName)"" ($($Member.Mail)) " -ForegroundColor Green
                }
                Write-Host 
            }
        }
        $SettingsObject = (Get-AzureADDirectorySetting | Where-Object -Property Displayname -Value "Group.Unified" -EQ)
        if (!$SettingsObject) {
            Write-Host "Create AzureAD directory settings for unified groups [Y/N] : " -NoNewline -ForegroundColor Magenta
            $Choice = Read-Host
            Write-Host
            if ($Choice.ToLower() -eq "y") {
                $Template = Get-AzureADDirectorySettingTemplate | Where-Object { $_.DisplayName -eq "Group.Unified" }
                $SettingsCopy = $Template.CreateDirectorySetting()
                # Security group can create security groups
                $IdGrp = $GrpSecu.ObjectId
                $EnGrp = "False"
                $SettingsCopy["EnableGroupCreation"] = $EnGrp
                $SettingsCopy["GroupCreationAllowedGroupId"] = $IdGrp
                Write-Host "GroupCreationAllowedGroupId=" -NoNewline
                Write-Host $IdGrp -ForegroundColor Yellow
                Write-Host "EnableGroupCreation=" -NoNewline
                Write-Host $EnGrp -ForegroundColor Yellow
                $SettingsObject = New-AzureADDirectorySetting -DirectorySetting $SettingsCopy
                $S = "AzureAD directory settings created for unified groups (Security group ""{0}"" can now create security groups)." -f $GroupName
                Write-Host $S
                log_message "OK" $S
            }
        }
        else {
            $IdGrp = $SettingsObject["GroupCreationAllowedGroupId"]
            $EnGrp = $SettingsObject["EnableGroupCreation"]
            Write-Host "AzureAD directory settings for unified groups : " -NoNewline
            Write-Host """$($SettingsObject.DisplayName)""" -ForegroundColor Yellow
            Write-Host "GroupCreationAllowedGroupId=" -NoNewline
            Write-Host $IdGrp -ForegroundColor Yellow
            Write-Host "EnableGroupCreation=" -NoNewline
            Write-Host $EnGrp -ForegroundColor Yellow
            if ($IdGrp -ne $null) {
                $FoundGrp = Get-AzureADGroup | Where { $_.ObjectId -eq $IdGrp } 
                if ($FoundGrp -ne $null) {
                    Write-Host "ObjectId $($IdGrp) resolved as ""$($FoundGrp.DisplayName)"""
                }
                else {
                    $S = "Group with Id=$($IdGrp) cannot be resolved !"
                    Write-Host $S -ForegroundColor Red              
                    log_message "ERROR" $S
                    $global:total_errors_count++  
                    Write-Host "Try to fix this problem [Y/N] ? : " -NoNewline -ForegroundColor Magenta
                    $Choice = Read-Host
                    Write-Host 
                    if ($Choice.ToLower() -eq "y") {
                        $SettingsObject["GroupCreationAllowedGroupId"] = $GrpSecu.ObjectId
                        $SettingsObject["EnableGroupCreation"] = "False"
                        Set-AzureADDirectorySetting -Id $SettingsObject.Id -DirectorySetting $SettingsObject
                        Write-Host "Security group " -NoNewline
                        Write-Host """$($GrpSecu.DisplayName)""" -NoNewline -ForegroundColor Yellow   
                        Write-Host " is allow to create security groups."
                        $S = "Setting ""GroupCreationAllowedGroupId"" changed to ""$($GrpSecu.ObjectId)""."
                        Write-Host $S -ForegroundColor Green
                        log_message "OK" $S
                    }
                }
            }
        }
        Write-Host
        Write-Host "Available options : "
        Write-Host "Delete the (s)ecurity group, D(i)sable/(E)nable security group, (D)elete AzureAD Directory Settings, E(x)it ? : " -NoNewline
        $Choice = Read-Host
        Write-Host 
        if ($Choice.ToLower() -eq "s") {
            Remove-AzureADGroup -ObjectId $GrpSecu.ObjectId
        }
        if ($Choice.ToLower() -eq "i") {
            $SettingsObject["GroupCreationAllowedGroupId"] = $null
            $SettingsObject["EnableGroupCreation"] = "True"
            Set-AzureADDirectorySetting -Id $SettingsObject.Id -DirectorySetting $SettingsObject
        }
        if ($Choice.ToLower() -eq "e") {
            $SettingsObject["GroupCreationAllowedGroupId"] = $GrpSecu.ObjectId
            $SettingsObject["EnableGroupCreation"] = "False"
            Set-AzureADDirectorySetting -Id $SettingsObject.Id -DirectorySetting $SettingsObject
        }
        if ($Choice.ToLower() -eq "d") {
            Remove-AzureADDirectorySetting -Id $SettingsObject.Id 
        }
        Disconnect-AzureAD
    }
    else {
        $global:total_errors_count++
        $S = "$($ModuleName) Powershell module cannot be loaded !"
        Write-Host $S -Foregroundcolor "Red"
        Log_Message "ERROR" $S
    }
}
log_total_errors_count